@@ -37,7 +37,8 @@ from warnings import warn
 
 import ipywidgets
 import ipywidgets as widgets
-from typing_extensions import Literal
+import typing_extensions
+from typing_extensions import Literal, Protocol
 
 from . import patch  # noqa: F401
 from . import _version
@@ -64,7 +65,10 @@ local = threading.local()
 T = TypeVar("T")
 U = TypeVar("U")
 W = TypeVar("W")  # used for widgets
+V = TypeVar("V")  # used for value type of widget
+V2 = TypeVar("V2")  # used for value type of widget
 E = TypeVar("E")  # used for elements
+P = typing_extensions.ParamSpec("P")
 
 WidgetOrList = Union[widgets.Widget, List[widgets.Widget]]
 EffectCleanupCallable = Callable[[], None]
@@ -212,7 +216,7 @@ class Element(Generic[W]):
 
         def format_kwarg(key, value):
             if key == "children":
-                if len(value) > 0:
+                if isinstance(value, (list, tuple)) and len(value) > 0:
                     contains_elements = any(isinstance(child, Element) for child in value)
                     if contains_elements:
                         return "children = ..."
@@ -358,6 +362,48 @@ class Element(Generic[W]):
         widget.unobserve(on_change, target_name)
 
 
+class Value(Generic[V], Protocol):
+    def get(self) -> V:
+        ...
+
+    def set(self, value: V):
+        ...
+
+
+class ValueElement(Generic[W, V], Element[W]):
+    def __init__(self, value_property, component, args=None, kwargs=None):
+        self.value_property = value_property
+        super().__init__(component, args, kwargs)
+
+    # TODO: we want to enable something like this, but requires a good hash function
+    # for the key
+    # def use_value(self, *default) -> V:
+    #     assert self.value_property in self.kwargs
+    #     default_value = self.kwargs[self.value_property]
+    #     # if not default:
+    #     # else:
+    #     #     default_value = default[0]
+    #     #     if "kwargs" in self.kwargs:
+    #     #         default_value = self.kwargs["value"]
+    #     value, set_value = use_state(default_value)
+    #     self.kwargs[self.value_property] = value
+    #     self.kwargs[f"on_{self.value_property}"] = lambda x: set_value(x)
+    #     return value
+
+    def connect(self, value: Value[V]):
+        """Will automatically set up a connection between the value and the widget.
+
+        The object passed should have a get and set method. The get method
+        set the current value, while the set method will be called when the
+        `on_value` event is triggered.
+
+        The get method could also be used to automically set up listeners.
+        """
+        self.kwargs[self.value_property] = value.get()
+        self.kwargs[f"on_{self.value_property}"] = lambda x: value.set(x)
+        return self
+
+
 FuncT = TypeVar("FuncT", bound=Callable[..., Element])
 
 
@@ -423,10 +469,11 @@ class ComponentWidget(Component):
 
 
 class ComponentFunction(Component):
-    def __init__(self, f: Callable[[], Element], mime_bundle=mime_bundle_default):
+    def __init__(self, f: Callable[[], Element], mime_bundle=mime_bundle_default, value_name=None):
         self.f = f
         self.name = self.f.__name__
         self.mime_bundle = mime_bundle
+        self.value_name = value_name
         functools.update_wrapper(self, f)
 
     def __eq__(self, rhs):
@@ -440,7 +487,10 @@ class ComponentFunction(Component):
         return f"react.component({self.f.__module__}.{self.f.__name__})"
 
     def __call__(self, *args, **kwargs):
-        el: Element = Element(self, args=args, kwargs=kwargs)
+        if self.value_name is not None:
+            el: Element = ValueElement(self.value_name, self, args=args, kwargs=kwargs)
+        else:
+            el = Element(self, args=args, kwargs=kwargs)
         el.mime_bundle = self.mime_bundle
         return el
 
@@ -473,6 +523,36 @@ def component(obj: FuncT = None, mime_bundle: Dict[str, Any] = mime_bundle_defau
         return wrapper
 
 
+@overload
+def value_component(
+    value_type: None, value_name="value", mime_bundle: Dict[str, Any] = mime_bundle_default
+) -> Callable[[Callable[P, ValueElement[W, V]]], Callable[P, ValueElement[W, V]]]:
+    ...
+
+
+@overload
+def value_component(
+    value_type: Type[V], value_name="value", mime_bundle: Dict[str, Any] = mime_bundle_default
+) -> Callable[[Callable[P, Element[W]]], Callable[P, ValueElement[W, V]]]:
+    ...
+
+
+def value_component(value_type: Union[Type[V], None], value_name="value", mime_bundle: Dict[str, Any] = mime_bundle_default):
+    """Creates a custom component that returns a ValueElement.
+
+    A ValueElement is a special element that can be used to connect to a Value, and be used to automatically
+    wire up the value and on_value events.
+    """
+
+    def wrapper(obj: Callable[P, Union[Element[W], ValueElement[W, V2]]]) -> Callable[P, ValueElement[W, V]]:
+        if isclass(obj) and issubclass(obj, widgets.Widget):
+            return cast(Callable[P, ValueElement[T, V]], ComponentWidget(widget=obj, mime_bundle=mime_bundle))
+        else:
+            return cast(Callable[P, ValueElement[T, V]], ComponentFunction(f=obj, mime_bundle=mime_bundle, value_name=value_name))
+
+    return wrapper
+
+
 def force_update():
     rc = _get_render_context()
     rc.force_update()
@@ -813,11 +893,13 @@ class _RenderContext:
         # for detecting stale elements used get_widget
         self._old_element_ids: Set[int] = set()
 
-    def _find(self, cls: Type[W], **matches):
+    def find(self, cls: Type[W] = ipywidgets.Widget, **matches):
         from .find import finder
 
         return finder(self).find(cls, **matches)
 
+    _find = find  # for backward compatibility
+
     def close(self):
         with self.thread_lock:
             self._closing = True
@@ -1375,13 +1457,12 @@ class _RenderContext:
                                 effect.next = None
                             else:
                                 # dependencies changed, cleanup and execute next
-                                if effect.cleanup:
-                                    try:
-                                        effect.cleanup()
-                                    except BaseException as e:
-                                        context.exceptions_self.append(e)
-                                        self._rerender_needed_reason = "Exception ocurred during effect"
-                                        self._rerender_needed = True
+                                try:
+                                    effect.cleanup()
+                                except BaseException as e:
+                                    context.exceptions_self.append(e)
+                                    self._rerender_needed_reason = "Exception ocurred during effect"
+                                    self._rerender_needed = True
                                 effect = child_context.effects[effect_index] = effect.next
                                 try:
                                     effect()
@@ -1585,13 +1666,12 @@ class _RenderContext:
                 self.context = child_context = context.children[key]
 
                 for effect_index, effect in enumerate(self.context.effects):
-                    if effect.cleanup:
-                        try:
-                            effect.cleanup()
-                        except BaseException as e:
-                            context.exceptions_self.append(e)
-                            self._rerender_needed_reason = "Exception ocurred during effect"
-                            self._rerender_needed = True
+                    try:
+                        effect.cleanup()
+                    except BaseException as e:
+                        context.exceptions_self.append(e)
+                        self._rerender_needed_reason = "Exception ocurred during effect"
+                        self._rerender_needed = True
                 assert self.context.root_element is not None
                 new_parent_key = join_key(parent_key, key)
                 self._remove_element(self.context.root_element, "/", parent_key=new_parent_key)
